{
    "$graph": [
        {
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement"
                }
            ],
            "class": "ExpressionTool",
            "inputs": [
                {
                    "type": "File",
                    "id": "#bam_inputs_decider.cwl/pe_bam_file"
                },
                {
                    "type": "int",
                    "id": "#bam_inputs_decider.cwl/pe_counts"
                },
                {
                    "type": "File",
                    "id": "#bam_inputs_decider.cwl/se_bam_file"
                },
                {
                    "type": "int",
                    "id": "#bam_inputs_decider.cwl/se_counts"
                }
            ],
            "expression": "${\n   var curr = [];\n   if(inputs.pe_counts > 0) {\n     curr.push(inputs.pe_bam_file)\n   }\n\n   if(inputs.se_counts > 0) {\n     curr.push(inputs.se_bam_file)\n   }\n   return {\"bam_files\": curr}\n }\n",
            "id": "#bam_inputs_decider.cwl",
            "outputs": [
                {
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "id": "#bam_inputs_decider.cwl/bam_files"
                }
            ]
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "quay.io/ncigdc/bio-client:latest"
                },
                {
                    "class": "InlineJavascriptRequirement"
                },
                {
                    "class": "ResourceRequirement",
                    "coresMin": 1,
                    "coresMax": 1,
                    "ramMin": 2000,
                    "ramMax": 2000,
                    "tmpdirMin": "$(Math.ceil (inputs.file_size / 1048576))",
                    "tmpdirMax": "$(Math.ceil (inputs.file_size / 1048576))",
                    "outdirMin": "$(Math.ceil (inputs.file_size / 1048576))",
                    "outdirMax": "$(Math.ceil (inputs.file_size / 1048576))"
                }
            ],
            "inputs": [
                {
                    "id": "#bioclient_download.cwl/config-file",
                    "type": "File",
                    "inputBinding": {
                        "prefix": "-c",
                        "position": 0
                    }
                },
                {
                    "id": "#bioclient_download.cwl/dir_path",
                    "type": "string",
                    "default": ".",
                    "inputBinding": {
                        "prefix": "--dir_path",
                        "position": 99
                    }
                },
                {
                    "id": "#bioclient_download.cwl/download",
                    "type": "string",
                    "default": "download",
                    "inputBinding": {
                        "position": 1
                    }
                },
                {
                    "id": "#bioclient_download.cwl/download_handle",
                    "type": "string",
                    "inputBinding": {
                        "position": 98
                    }
                },
                {
                    "id": "#bioclient_download.cwl/file_size",
                    "type": "long",
                    "default": 1
                }
            ],
            "outputs": [
                {
                    "id": "#bioclient_download.cwl/output",
                    "type": "File",
                    "outputBinding": {
                        "glob": "*"
                    }
                }
            ],
            "baseCommand": [
                "/usr/local/bin/bio_client.py"
            ],
            "id": "#bioclient_download.cwl"
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "quay.io/ncigdc/bio-client:latest"
                },
                {
                    "class": "InlineJavascriptRequirement"
                },
                {
                    "class": "ResourceRequirement",
                    "coresMin": 1,
                    "coresMax": 1,
                    "ramMin": 1000,
                    "ramMax": 1000,
                    "tmpdirMin": 1,
                    "tmpdirMax": 1,
                    "outdirMin": 1,
                    "outdirMax": 1
                }
            ],
            "inputs": [
                {
                    "id": "#bioclient_upload_pull_uuid.cwl/config-file",
                    "type": "File",
                    "inputBinding": {
                        "prefix": "--config-file",
                        "position": 0
                    }
                },
                {
                    "id": "#bioclient_upload_pull_uuid.cwl/upload",
                    "type": "string",
                    "default": "upload",
                    "inputBinding": {
                        "position": 1
                    }
                },
                {
                    "id": "#bioclient_upload_pull_uuid.cwl/upload-bucket",
                    "type": "string",
                    "inputBinding": {
                        "prefix": "--upload-bucket",
                        "position": 2
                    }
                },
                {
                    "id": "#bioclient_upload_pull_uuid.cwl/upload-key",
                    "type": "string",
                    "inputBinding": {
                        "prefix": "--upload_key",
                        "position": 3
                    }
                },
                {
                    "id": "#bioclient_upload_pull_uuid.cwl/input",
                    "type": "File",
                    "inputBinding": {
                        "position": 99
                    }
                }
            ],
            "outputs": [
                {
                    "id": "#bioclient_upload_pull_uuid.cwl/output",
                    "type": "File",
                    "outputBinding": {
                        "glob": "*_upload.json"
                    }
                },
                {
                    "id": "#bioclient_upload_pull_uuid.cwl/uuid",
                    "type": "string",
                    "outputBinding": {
                        "glob": "*_upload.json",
                        "loadContents": true,
                        "outputEval": "${\n   var data = JSON.parse(self[0].contents);\n   return(data[\"did\"]);\n }\n"
                    }
                }
            ],
            "baseCommand": [
                "/usr/local/bin/bio_client.py"
            ],
            "id": "#bioclient_upload_pull_uuid.cwl"
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "quay.io/ncigdc/htseq-tool:2a8b0bfec9dc52aaeac29ee8e7ff3284e79553c0"
                },
                {
                    "class": "InlineJavascriptRequirement",
                    "expressionLib": [
                        "function sum_file_array_size(farr) {\n  var divisor = 1048576;\n  var total = 0;\n  for (var i = 0; i<farr.length; i++) {\n    total += farr[i].size\n  } \n  return Math.ceil(total / divisor);\n} \n",
                        "function file_size_multiplier(fobj, mult) {\n  mult = typeof mult !== 'undefined' ? mult : 1;\n  var divisor = 1048576;\n  return Math.ceil(mult * fobj.size / divisor);\n} \n"
                    ]
                },
                {
                    "class": "ResourceRequirement",
                    "coresMin": 1,
                    "ramMin": 500,
                    "tmpdirMin": "$(sum_file_array_size([inputs.htseq_file, inputs.aggregate_length_file]) * 2)",
                    "outdirMin": "$(sum_file_array_size([inputs.htseq_file, inputs.aggregate_length_file]) * 2)"
                }
            ],
            "inputs": [
                {
                    "type": "File",
                    "inputBinding": {
                        "prefix": "--aggregate_length_file"
                    },
                    "id": "#calculate_fpkm.cwl/aggregate_length_file"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "prefix": "--htseq_counts"
                    },
                    "id": "#calculate_fpkm.cwl/htseq_file"
                },
                {
                    "type": "string",
                    "inputBinding": {
                        "prefix": "--output_prefix"
                    },
                    "id": "#calculate_fpkm.cwl/output_prefix"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(inputs.output_prefix + '.FPKM.txt.gz')"
                    },
                    "id": "#calculate_fpkm.cwl/fpkm"
                },
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(inputs.output_prefix + '.FPKM-UQ.txt.gz')"
                    },
                    "id": "#calculate_fpkm.cwl/fpkm_uq"
                }
            ],
            "baseCommand": [
                "/opt/htseq-tools/venv/bin/htseq-tools",
                "fpkm"
            ],
            "id": "#calculate_fpkm.cwl"
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "quay.io/ncigdc/htseq-tool:2a8b0bfec9dc52aaeac29ee8e7ff3284e79553c0"
                },
                {
                    "class": "InlineJavascriptRequirement",
                    "expressionLib": [
                        "function sum_file_array_size(farr) {\n  var divisor = 1048576;\n  var total = 0;\n  for (var i = 0; i<farr.length; i++) {\n    total += farr[i].size\n  } \n  return Math.ceil(total / divisor);\n} \n",
                        "function file_size_multiplier(fobj, mult) {\n  mult = typeof mult !== 'undefined' ? mult : 1;\n  var divisor = 1048576;\n  return Math.ceil(mult * fobj.size / divisor);\n} \n"
                    ]
                },
                {
                    "class": "ResourceRequirement",
                    "coresMin": 1,
                    "ramMin": 1000,
                    "tmpdirMin": "$(sum_file_array_size([inputs.bam_file, inputs.gtf_file]) * 1.1)",
                    "outdirMin": "$(sum_file_array_size([inputs.bam_file, inputs.gtf_file]) * 1.1)"
                }
            ],
            "inputs": [
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 7
                    },
                    "id": "#htseq_count.cwl/bam_file"
                },
                {
                    "type": "string",
                    "default": "exon",
                    "inputBinding": {
                        "prefix": "-t",
                        "position": 4
                    },
                    "id": "#htseq_count.cwl/feature_type"
                },
                {
                    "type": "string",
                    "default": "bam",
                    "inputBinding": {
                        "prefix": "-f",
                        "position": 0
                    },
                    "id": "#htseq_count.cwl/file_format"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 8
                    },
                    "id": "#htseq_count.cwl/gtf_file"
                },
                {
                    "type": "string",
                    "default": "gene_id",
                    "inputBinding": {
                        "prefix": "-i",
                        "position": 5
                    },
                    "id": "#htseq_count.cwl/id_attr"
                },
                {
                    "type": "int",
                    "default": 10,
                    "inputBinding": {
                        "prefix": "-a",
                        "position": 3
                    },
                    "id": "#htseq_count.cwl/min_qual"
                },
                {
                    "type": "string",
                    "default": "intersection-nonempty",
                    "inputBinding": {
                        "prefix": "-m",
                        "position": 6
                    },
                    "id": "#htseq_count.cwl/mode"
                },
                {
                    "type": "string",
                    "id": "#htseq_count.cwl/output_filename"
                },
                {
                    "type": "string",
                    "default": "pos",
                    "inputBinding": {
                        "prefix": "-r",
                        "position": 1
                    },
                    "id": "#htseq_count.cwl/sort_order"
                },
                {
                    "type": "string",
                    "default": "no",
                    "inputBinding": {
                        "prefix": "-s",
                        "position": 2
                    },
                    "id": "#htseq_count.cwl/stranded"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "doc": "raw counts file",
                    "outputBinding": {
                        "glob": "$(inputs.output_filename)"
                    },
                    "id": "#htseq_count.cwl/counts"
                }
            ],
            "stdout": "$(inputs.output_filename)",
            "baseCommand": [
                "htseq-count"
            ],
            "id": "#htseq_count.cwl"
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "quay.io/ncigdc/htseq-tool:2a8b0bfec9dc52aaeac29ee8e7ff3284e79553c0"
                },
                {
                    "class": "InlineJavascriptRequirement",
                    "expressionLib": [
                        "function sum_file_array_size(farr) {\n  var divisor = 1048576;\n  var total = 0;\n  for (var i = 0; i<farr.length; i++) {\n    total += farr[i].size\n  } \n  return Math.ceil(total / divisor);\n} \n",
                        "function file_size_multiplier(fobj, mult) {\n  mult = typeof mult !== 'undefined' ? mult : 1;\n  var divisor = 1048576;\n  return Math.ceil(mult * fobj.size / divisor);\n} \n"
                    ]
                },
                {
                    "class": "ResourceRequirement",
                    "coresMin": 1,
                    "ramMin": 500,
                    "tmpdirMin": "$(sum_file_array_size(inputs.htseq_file) * 1.1)",
                    "outdirMin": "$(sum_file_array_size(inputs.htseq_file) * 1.1)"
                }
            ],
            "inputs": [
                {
                    "type": {
                        "type": "array",
                        "items": "File",
                        "inputBinding": {
                            "prefix": "--htseq_counts"
                        }
                    },
                    "id": "#merge_htseq.cwl/htseq_file"
                },
                {
                    "type": "string",
                    "inputBinding": {
                        "prefix": "--out_file"
                    },
                    "id": "#merge_htseq.cwl/output_filename"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "doc": "merged counts file",
                    "outputBinding": {
                        "glob": "$(inputs.output_filename)"
                    },
                    "id": "#merge_htseq.cwl/counts"
                }
            ],
            "baseCommand": [
                "/opt/htseq-tools/venv/bin/htseq-tools",
                "merge_counts"
            ],
            "id": "#merge_htseq.cwl"
        },
        {
            "requirements": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "quay.io/ncigdc/samtools:147bd4cc606a63c7435907d97fea6e94e9ea9ed58c18f390cab8bc40b1992df7"
                },
                {
                    "class": "InlineJavascriptRequirement",
                    "expressionLib": [
                        "function sum_file_array_size(farr) {\n  var divisor = 1048576;\n  var total = 0;\n  for (var i = 0; i<farr.length; i++) {\n    total += farr[i].size\n  } \n  return Math.ceil(total / divisor);\n} \n",
                        "function file_size_multiplier(fobj, mult) {\n  mult = typeof mult !== 'undefined' ? mult : 1;\n  var divisor = 1048576;\n  return Math.ceil(mult * fobj.size / divisor);\n} \n"
                    ]
                },
                {
                    "class": "ResourceRequirement",
                    "coresMin": "$(inputs.threads ? inputs.threads : 1)",
                    "ramMin": 1000,
                    "tmpdirMin": "$(file_size_multiplier(inputs.bam_file, 1.5))",
                    "outdirMin": "$(file_size_multiplier(inputs.bam_file, 1.5))"
                }
            ],
            "class": "CommandLineTool",
            "inputs": [
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 4
                    },
                    "id": "#samtools_filter_bam.cwl/bam_file"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 3,
                        "prefix": "-F"
                    },
                    "id": "#samtools_filter_bam.cwl/exclude_flag"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 2,
                        "prefix": "-f"
                    },
                    "id": "#samtools_filter_bam.cwl/include_flag"
                },
                {
                    "type": "string",
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-o"
                    },
                    "id": "#samtools_filter_bam.cwl/output_filename"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 0,
                        "prefix": "-@"
                    },
                    "id": "#samtools_filter_bam.cwl/threads"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(inputs.output_filename)"
                    },
                    "id": "#samtools_filter_bam.cwl/output"
                }
            ],
            "baseCommand": [
                "samtools",
                "view",
                "-b"
            ],
            "id": "#samtools_filter_bam.cwl"
        },
        {
            "requirements": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "quay.io/ncigdc/samtools:147bd4cc606a63c7435907d97fea6e94e9ea9ed58c18f390cab8bc40b1992df7"
                },
                {
                    "class": "InlineJavascriptRequirement",
                    "expressionLib": [
                        "function sum_file_array_size(farr) {\n  var divisor = 1048576;\n  var total = 0;\n  for (var i = 0; i<farr.length; i++) {\n    total += farr[i].size\n  } \n  return Math.ceil(total / divisor);\n} \n",
                        "function file_size_multiplier(fobj, mult) {\n  mult = typeof mult !== 'undefined' ? mult : 1;\n  var divisor = 1048576;\n  return Math.ceil(mult * fobj.size / divisor);\n} \n"
                    ]
                },
                {
                    "class": "ResourceRequirement",
                    "coresMin": "$(inputs.threads ? inputs.threads : 1)",
                    "ramMin": 1000,
                    "tmpdirMin": "$(file_size_multiplier(inputs.bam_file, 1.5))",
                    "outdirMin": "$(file_size_multiplier(inputs.bam_file, 1.5))"
                }
            ],
            "class": "CommandLineTool",
            "inputs": [
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 4
                    },
                    "id": "#samtools_namesort_bam.cwl/bam_file"
                },
                {
                    "type": "string",
                    "default": "768M",
                    "inputBinding": {
                        "position": 2,
                        "prefix": "-m"
                    },
                    "id": "#samtools_namesort_bam.cwl/max_mem"
                },
                {
                    "type": "string",
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-o"
                    },
                    "id": "#samtools_namesort_bam.cwl/output_filename"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 0,
                        "prefix": "-@"
                    },
                    "id": "#samtools_namesort_bam.cwl/threads"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 3,
                        "prefix": "-T"
                    },
                    "id": "#samtools_namesort_bam.cwl/tmp_prefix"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(inputs.output_filename)"
                    },
                    "id": "#samtools_namesort_bam.cwl/output"
                }
            ],
            "baseCommand": [
                "samtools",
                "sort",
                "-n"
            ],
            "id": "#samtools_namesort_bam.cwl"
        },
        {
            "requirements": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "quay.io/ncigdc/samtools:147bd4cc606a63c7435907d97fea6e94e9ea9ed58c18f390cab8bc40b1992df7"
                },
                {
                    "class": "ShellCommandRequirement"
                },
                {
                    "class": "InlineJavascriptRequirement",
                    "expressionLib": [
                        "function sum_file_array_size(farr) {\n  var divisor = 1048576;\n  var total = 0;\n  for (var i = 0; i<farr.length; i++) {\n    total += farr[i].size\n  } \n  return Math.ceil(total / divisor);\n} \n",
                        "function file_size_multiplier(fobj, mult) {\n  mult = typeof mult !== 'undefined' ? mult : 1;\n  var divisor = 1048576;\n  return Math.ceil(mult * fobj.size / divisor);\n} \n"
                    ]
                },
                {
                    "class": "ResourceRequirement",
                    "coresMin": 1,
                    "ramMin": 100,
                    "tmpdirMin": "$(file_size_multiplier(inputs.bam_file))",
                    "outdirMin": "$(file_size_multiplier(inputs.bam_file))"
                }
            ],
            "class": "CommandLineTool",
            "inputs": [
                {
                    "type": "File",
                    "id": "#samtools_view_head_check.cwl/bam_file"
                }
            ],
            "outputs": [
                {
                    "type": "int",
                    "outputBinding": {
                        "glob": "counts",
                        "loadContents": true,
                        "outputEval": "${\n   return parseInt(self[0].contents)\n }\n"
                    },
                    "id": "#samtools_view_head_check.cwl/counts"
                }
            ],
            "stdout": "counts",
            "baseCommand": [],
            "arguments": [
                {
                    "valueFrom": "${\n    var curr = [\"samtools\", \"view\", inputs.bam_file, \"|\", \"head\", \"|\", \"wc\", \"-l\"]\n    return curr\n }\n",
                    "position": 0,
                    "shellQuote": false
                }
            ],
            "id": "#samtools_view_head_check.cwl"
        },
        {
            "class": "Workflow",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement"
                },
                {
                    "class": "StepInputExpressionRequirement"
                },
                {
                    "class": "SubworkflowFeatureRequirement"
                },
                {
                    "class": "MultipleInputFeatureRequirement"
                }
            ],
            "inputs": [
                {
                    "type": "string",
                    "id": "#main/bam_uuid"
                },
                {
                    "type": "File",
                    "id": "#main/bioclient_config"
                },
                {
                    "type": "string",
                    "id": "#main/gene_lengths_uuid"
                },
                {
                    "type": "string",
                    "id": "#main/gtf_uuid"
                },
                {
                    "type": "string",
                    "id": "#main/job_uuid"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "id": "#main/threads"
                },
                {
                    "type": "string",
                    "id": "#main/upload_bucket"
                }
            ],
            "outputs": [
                {
                    "type": "string",
                    "outputSource": "#main/run_upload_workflow/counts_uuid",
                    "id": "#main/htseq_counts_uuid"
                },
                {
                    "type": "string",
                    "outputSource": "#main/run_upload_workflow/fpkm_uq_uuid",
                    "id": "#main/htseq_fpkm_uq_uuid"
                },
                {
                    "type": "string",
                    "outputSource": "#main/run_upload_workflow/fpkm_uuid",
                    "id": "#main/htseq_fpkm_uuid"
                }
            ],
            "steps": [
                {
                    "run": "#htseq_workflow.cwl",
                    "in": [
                        {
                            "source": "#main/run_split_bam/bam_list",
                            "id": "#main/run_htseq_workflow/bam_file"
                        },
                        {
                            "source": "#main/run_stage_workflow/gene_lengths",
                            "id": "#main/run_htseq_workflow/gene_lengths"
                        },
                        {
                            "source": "#main/run_stage_workflow/gtf",
                            "id": "#main/run_htseq_workflow/gtf_file"
                        },
                        {
                            "source": "#main/job_uuid",
                            "id": "#main/run_htseq_workflow/job_uuid"
                        }
                    ],
                    "out": [
                        "#main/run_htseq_workflow/htseq_counts",
                        "#main/run_htseq_workflow/htseq_fpkm",
                        "#main/run_htseq_workflow/htseq_fpkm_uq"
                    ],
                    "id": "#main/run_htseq_workflow"
                },
                {
                    "run": "#split_bam_workflow.cwl",
                    "in": [
                        {
                            "source": "#main/run_stage_workflow/bam",
                            "id": "#main/run_split_bam/bam_file"
                        },
                        {
                            "source": "#main/job_uuid",
                            "id": "#main/run_split_bam/job_uuid"
                        },
                        {
                            "source": "#main/threads",
                            "id": "#main/run_split_bam/threads"
                        }
                    ],
                    "out": [
                        "#main/run_split_bam/bam_list"
                    ],
                    "id": "#main/run_split_bam"
                },
                {
                    "run": "#stage_data_workflow.cwl",
                    "in": [
                        {
                            "source": "#main/bam_uuid",
                            "id": "#main/run_stage_workflow/bam_uuid"
                        },
                        {
                            "source": "#main/bioclient_config",
                            "id": "#main/run_stage_workflow/bioclient_config"
                        },
                        {
                            "source": "#main/gene_lengths_uuid",
                            "id": "#main/run_stage_workflow/gene_lengths_uuid"
                        },
                        {
                            "source": "#main/gtf_uuid",
                            "id": "#main/run_stage_workflow/gtf_uuid"
                        }
                    ],
                    "out": [
                        "#main/run_stage_workflow/gtf",
                        "#main/run_stage_workflow/gene_lengths",
                        "#main/run_stage_workflow/bam"
                    ],
                    "id": "#main/run_stage_workflow"
                },
                {
                    "run": "#upload_results_workflow.cwl",
                    "in": [
                        {
                            "source": "#main/bioclient_config",
                            "id": "#main/run_upload_workflow/bioclient_config"
                        },
                        {
                            "source": "#main/run_htseq_workflow/htseq_counts",
                            "id": "#main/run_upload_workflow/htseq_counts_file"
                        },
                        {
                            "source": "#main/run_htseq_workflow/htseq_fpkm",
                            "id": "#main/run_upload_workflow/htseq_fpkm_file"
                        },
                        {
                            "source": "#main/run_htseq_workflow/htseq_fpkm_uq",
                            "id": "#main/run_upload_workflow/htseq_fpkm_uq_file"
                        },
                        {
                            "source": "#main/job_uuid",
                            "id": "#main/run_upload_workflow/job_uuid"
                        },
                        {
                            "source": "#main/upload_bucket",
                            "id": "#main/run_upload_workflow/upload_bucket"
                        }
                    ],
                    "out": [
                        "#main/run_upload_workflow/counts_uuid",
                        "#main/run_upload_workflow/fpkm_uuid",
                        "#main/run_upload_workflow/fpkm_uq_uuid"
                    ],
                    "id": "#main/run_upload_workflow"
                }
            ],
            "id": "#main"
        },
        {
            "class": "Workflow",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement"
                },
                {
                    "class": "StepInputExpressionRequirement"
                },
                {
                    "class": "ScatterFeatureRequirement"
                }
            ],
            "inputs": [
                {
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "id": "#htseq_workflow.cwl/bam_file"
                },
                {
                    "type": "File",
                    "id": "#htseq_workflow.cwl/gene_lengths"
                },
                {
                    "type": "File",
                    "id": "#htseq_workflow.cwl/gtf_file"
                },
                {
                    "type": "string",
                    "id": "#htseq_workflow.cwl/job_uuid"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputSource": "#htseq_workflow.cwl/run_merge_counts/counts",
                    "id": "#htseq_workflow.cwl/htseq_counts"
                },
                {
                    "type": "File",
                    "outputSource": "#htseq_workflow.cwl/run_fpkm/fpkm",
                    "id": "#htseq_workflow.cwl/htseq_fpkm"
                },
                {
                    "type": "File",
                    "outputSource": "#htseq_workflow.cwl/run_fpkm/fpkm_uq",
                    "id": "#htseq_workflow.cwl/htseq_fpkm_uq"
                }
            ],
            "steps": [
                {
                    "run": "#calculate_fpkm.cwl",
                    "in": [
                        {
                            "source": "#htseq_workflow.cwl/gene_lengths",
                            "id": "#htseq_workflow.cwl/run_fpkm/aggregate_length_file"
                        },
                        {
                            "source": "#htseq_workflow.cwl/run_merge_counts/counts",
                            "id": "#htseq_workflow.cwl/run_fpkm/htseq_file"
                        },
                        {
                            "source": "#htseq_workflow.cwl/job_uuid",
                            "id": "#htseq_workflow.cwl/run_fpkm/output_prefix"
                        }
                    ],
                    "out": [
                        "#htseq_workflow.cwl/run_fpkm/fpkm",
                        "#htseq_workflow.cwl/run_fpkm/fpkm_uq"
                    ],
                    "id": "#htseq_workflow.cwl/run_fpkm"
                },
                {
                    "run": "#htseq_count.cwl",
                    "scatter": [
                        "#htseq_workflow.cwl/run_htseq_counts/bam_file",
                        "#htseq_workflow.cwl/run_htseq_counts/output_filename"
                    ],
                    "scatterMethod": "dotproduct",
                    "in": [
                        {
                            "source": "#htseq_workflow.cwl/bam_file",
                            "id": "#htseq_workflow.cwl/run_htseq_counts/bam_file"
                        },
                        {
                            "source": "#htseq_workflow.cwl/gtf_file",
                            "id": "#htseq_workflow.cwl/run_htseq_counts/gtf_file"
                        },
                        {
                            "source": "#htseq_workflow.cwl/bam_file",
                            "valueFrom": "$(self.nameroot + '.htseq_counts.txt')",
                            "id": "#htseq_workflow.cwl/run_htseq_counts/output_filename"
                        },
                        {
                            "default": "name",
                            "id": "#htseq_workflow.cwl/run_htseq_counts/sort_order"
                        }
                    ],
                    "out": [
                        "#htseq_workflow.cwl/run_htseq_counts/counts"
                    ],
                    "id": "#htseq_workflow.cwl/run_htseq_counts"
                },
                {
                    "run": "#merge_htseq.cwl",
                    "in": [
                        {
                            "source": "#htseq_workflow.cwl/run_htseq_counts/counts",
                            "id": "#htseq_workflow.cwl/run_merge_counts/htseq_file"
                        },
                        {
                            "source": "#htseq_workflow.cwl/job_uuid",
                            "valueFrom": "$(self + '.htseq_counts.txt.gz')",
                            "id": "#htseq_workflow.cwl/run_merge_counts/output_filename"
                        }
                    ],
                    "out": [
                        "#htseq_workflow.cwl/run_merge_counts/counts"
                    ],
                    "id": "#htseq_workflow.cwl/run_merge_counts"
                }
            ],
            "id": "#htseq_workflow.cwl"
        },
        {
            "class": "Workflow",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement"
                },
                {
                    "class": "StepInputExpressionRequirement"
                },
                {
                    "class": "MultipleInputFeatureRequirement"
                }
            ],
            "inputs": [
                {
                    "type": "File",
                    "id": "#split_bam_workflow.cwl/bam_file"
                },
                {
                    "type": "string",
                    "id": "#split_bam_workflow.cwl/job_uuid"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "id": "#split_bam_workflow.cwl/threads"
                }
            ],
            "outputs": [
                {
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "outputSource": "#split_bam_workflow.cwl/run_inputs_decider/bam_files",
                    "id": "#split_bam_workflow.cwl/bam_list"
                }
            ],
            "steps": [
                {
                    "run": "#samtools_filter_bam.cwl",
                    "in": [
                        {
                            "source": "#split_bam_workflow.cwl/run_namesort_bam/output",
                            "id": "#split_bam_workflow.cwl/run_get_pe/bam_file"
                        },
                        {
                            "default": 4,
                            "id": "#split_bam_workflow.cwl/run_get_pe/exclude_flag"
                        },
                        {
                            "default": 1,
                            "id": "#split_bam_workflow.cwl/run_get_pe/include_flag"
                        },
                        {
                            "source": "#split_bam_workflow.cwl/job_uuid",
                            "valueFrom": "$(self + '.paired.bam')",
                            "id": "#split_bam_workflow.cwl/run_get_pe/output_filename"
                        },
                        {
                            "source": "#split_bam_workflow.cwl/threads",
                            "id": "#split_bam_workflow.cwl/run_get_pe/threads"
                        }
                    ],
                    "out": [
                        "#split_bam_workflow.cwl/run_get_pe/output"
                    ],
                    "id": "#split_bam_workflow.cwl/run_get_pe"
                },
                {
                    "run": "#samtools_filter_bam.cwl",
                    "in": [
                        {
                            "source": "#split_bam_workflow.cwl/run_namesort_bam/output",
                            "id": "#split_bam_workflow.cwl/run_get_se/bam_file"
                        },
                        {
                            "default": 5,
                            "id": "#split_bam_workflow.cwl/run_get_se/exclude_flag"
                        },
                        {
                            "source": "#split_bam_workflow.cwl/job_uuid",
                            "valueFrom": "$(self + '.unpaired.bam')",
                            "id": "#split_bam_workflow.cwl/run_get_se/output_filename"
                        },
                        {
                            "source": "#split_bam_workflow.cwl/threads",
                            "id": "#split_bam_workflow.cwl/run_get_se/threads"
                        }
                    ],
                    "out": [
                        "#split_bam_workflow.cwl/run_get_se/output"
                    ],
                    "id": "#split_bam_workflow.cwl/run_get_se"
                },
                {
                    "run": "#bam_inputs_decider.cwl",
                    "in": [
                        {
                            "source": "#split_bam_workflow.cwl/run_get_pe/output",
                            "id": "#split_bam_workflow.cwl/run_inputs_decider/pe_bam_file"
                        },
                        {
                            "source": "#split_bam_workflow.cwl/run_pe_check/counts",
                            "id": "#split_bam_workflow.cwl/run_inputs_decider/pe_counts"
                        },
                        {
                            "source": "#split_bam_workflow.cwl/run_get_se/output",
                            "id": "#split_bam_workflow.cwl/run_inputs_decider/se_bam_file"
                        },
                        {
                            "source": "#split_bam_workflow.cwl/run_se_check/counts",
                            "id": "#split_bam_workflow.cwl/run_inputs_decider/se_counts"
                        }
                    ],
                    "out": [
                        "#split_bam_workflow.cwl/run_inputs_decider/bam_files"
                    ],
                    "id": "#split_bam_workflow.cwl/run_inputs_decider"
                },
                {
                    "run": "#samtools_namesort_bam.cwl",
                    "in": [
                        {
                            "source": "#split_bam_workflow.cwl/bam_file",
                            "id": "#split_bam_workflow.cwl/run_namesort_bam/bam_file"
                        },
                        {
                            "source": "#split_bam_workflow.cwl/job_uuid",
                            "valueFrom": "$(self + '.namesort.bam')",
                            "id": "#split_bam_workflow.cwl/run_namesort_bam/output_filename"
                        },
                        {
                            "source": "#split_bam_workflow.cwl/threads",
                            "id": "#split_bam_workflow.cwl/run_namesort_bam/threads"
                        },
                        {
                            "source": "#split_bam_workflow.cwl/job_uuid",
                            "id": "#split_bam_workflow.cwl/run_namesort_bam/tmp_prefix"
                        }
                    ],
                    "out": [
                        "#split_bam_workflow.cwl/run_namesort_bam/output"
                    ],
                    "id": "#split_bam_workflow.cwl/run_namesort_bam"
                },
                {
                    "run": "#samtools_view_head_check.cwl",
                    "in": [
                        {
                            "source": "#split_bam_workflow.cwl/run_get_pe/output",
                            "id": "#split_bam_workflow.cwl/run_pe_check/bam_file"
                        }
                    ],
                    "out": [
                        "#split_bam_workflow.cwl/run_pe_check/counts"
                    ],
                    "id": "#split_bam_workflow.cwl/run_pe_check"
                },
                {
                    "run": "#samtools_view_head_check.cwl",
                    "in": [
                        {
                            "source": "#split_bam_workflow.cwl/run_get_se/output",
                            "id": "#split_bam_workflow.cwl/run_se_check/bam_file"
                        }
                    ],
                    "out": [
                        "#split_bam_workflow.cwl/run_se_check/counts"
                    ],
                    "id": "#split_bam_workflow.cwl/run_se_check"
                }
            ],
            "id": "#split_bam_workflow.cwl"
        },
        {
            "class": "Workflow",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement"
                }
            ],
            "inputs": [
                {
                    "type": "string",
                    "id": "#stage_data_workflow.cwl/bam_uuid"
                },
                {
                    "type": "File",
                    "id": "#stage_data_workflow.cwl/bioclient_config"
                },
                {
                    "type": "string",
                    "id": "#stage_data_workflow.cwl/gene_lengths_uuid"
                },
                {
                    "type": "string",
                    "id": "#stage_data_workflow.cwl/gtf_uuid"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputSource": "#stage_data_workflow.cwl/extract_bam/output",
                    "id": "#stage_data_workflow.cwl/bam"
                },
                {
                    "type": "File",
                    "outputSource": "#stage_data_workflow.cwl/extract_gene_lengths/output",
                    "id": "#stage_data_workflow.cwl/gene_lengths"
                },
                {
                    "type": "File",
                    "outputSource": "#stage_data_workflow.cwl/extract_gtf/output",
                    "id": "#stage_data_workflow.cwl/gtf"
                }
            ],
            "steps": [
                {
                    "run": "#bioclient_download.cwl",
                    "in": [
                        {
                            "source": "#stage_data_workflow.cwl/bioclient_config",
                            "id": "#stage_data_workflow.cwl/extract_bam/config-file"
                        },
                        {
                            "source": "#stage_data_workflow.cwl/bam_uuid",
                            "id": "#stage_data_workflow.cwl/extract_bam/download_handle"
                        }
                    ],
                    "out": [
                        "#stage_data_workflow.cwl/extract_bam/output"
                    ],
                    "id": "#stage_data_workflow.cwl/extract_bam"
                },
                {
                    "run": "#bioclient_download.cwl",
                    "in": [
                        {
                            "source": "#stage_data_workflow.cwl/bioclient_config",
                            "id": "#stage_data_workflow.cwl/extract_gene_lengths/config-file"
                        },
                        {
                            "source": "#stage_data_workflow.cwl/gene_lengths_uuid",
                            "id": "#stage_data_workflow.cwl/extract_gene_lengths/download_handle"
                        }
                    ],
                    "out": [
                        "#stage_data_workflow.cwl/extract_gene_lengths/output"
                    ],
                    "id": "#stage_data_workflow.cwl/extract_gene_lengths"
                },
                {
                    "run": "#bioclient_download.cwl",
                    "in": [
                        {
                            "source": "#stage_data_workflow.cwl/bioclient_config",
                            "id": "#stage_data_workflow.cwl/extract_gtf/config-file"
                        },
                        {
                            "source": "#stage_data_workflow.cwl/gtf_uuid",
                            "id": "#stage_data_workflow.cwl/extract_gtf/download_handle"
                        }
                    ],
                    "out": [
                        "#stage_data_workflow.cwl/extract_gtf/output"
                    ],
                    "id": "#stage_data_workflow.cwl/extract_gtf"
                }
            ],
            "id": "#stage_data_workflow.cwl"
        },
        {
            "class": "Workflow",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement"
                },
                {
                    "class": "StepInputExpressionRequirement"
                }
            ],
            "inputs": [
                {
                    "type": "File",
                    "id": "#upload_results_workflow.cwl/bioclient_config"
                },
                {
                    "type": "File",
                    "id": "#upload_results_workflow.cwl/htseq_counts_file"
                },
                {
                    "type": "File",
                    "id": "#upload_results_workflow.cwl/htseq_fpkm_file"
                },
                {
                    "type": "File",
                    "id": "#upload_results_workflow.cwl/htseq_fpkm_uq_file"
                },
                {
                    "type": "string",
                    "id": "#upload_results_workflow.cwl/job_uuid"
                },
                {
                    "type": "string",
                    "id": "#upload_results_workflow.cwl/upload_bucket"
                }
            ],
            "outputs": [
                {
                    "type": "string",
                    "outputSource": "#upload_results_workflow.cwl/upload_counts/uuid",
                    "id": "#upload_results_workflow.cwl/counts_uuid"
                },
                {
                    "type": "string",
                    "outputSource": "#upload_results_workflow.cwl/upload_fpkm_uq/uuid",
                    "id": "#upload_results_workflow.cwl/fpkm_uq_uuid"
                },
                {
                    "type": "string",
                    "outputSource": "#upload_results_workflow.cwl/upload_fpkm/uuid",
                    "id": "#upload_results_workflow.cwl/fpkm_uuid"
                }
            ],
            "steps": [
                {
                    "run": "#bioclient_upload_pull_uuid.cwl",
                    "in": [
                        {
                            "source": "#upload_results_workflow.cwl/bioclient_config",
                            "id": "#upload_results_workflow.cwl/upload_counts/config-file"
                        },
                        {
                            "source": "#upload_results_workflow.cwl/htseq_counts_file",
                            "id": "#upload_results_workflow.cwl/upload_counts/input"
                        },
                        {
                            "source": "#upload_results_workflow.cwl/upload_bucket",
                            "id": "#upload_results_workflow.cwl/upload_counts/upload-bucket"
                        },
                        {
                            "source": [
                                "#upload_results_workflow.cwl/job_uuid",
                                "#upload_results_workflow.cwl/htseq_counts_file"
                            ],
                            "valueFrom": "$(self[0] + '/' + self[1].basename)",
                            "id": "#upload_results_workflow.cwl/upload_counts/upload-key"
                        }
                    ],
                    "out": [
                        "#upload_results_workflow.cwl/upload_counts/output",
                        "#upload_results_workflow.cwl/upload_counts/uuid"
                    ],
                    "id": "#upload_results_workflow.cwl/upload_counts"
                },
                {
                    "run": "#bioclient_upload_pull_uuid.cwl",
                    "in": [
                        {
                            "source": "#upload_results_workflow.cwl/bioclient_config",
                            "id": "#upload_results_workflow.cwl/upload_fpkm/config-file"
                        },
                        {
                            "source": "#upload_results_workflow.cwl/htseq_fpkm_file",
                            "id": "#upload_results_workflow.cwl/upload_fpkm/input"
                        },
                        {
                            "source": "#upload_results_workflow.cwl/upload_bucket",
                            "id": "#upload_results_workflow.cwl/upload_fpkm/upload-bucket"
                        },
                        {
                            "source": [
                                "#upload_results_workflow.cwl/job_uuid",
                                "#upload_results_workflow.cwl/htseq_fpkm_file"
                            ],
                            "valueFrom": "$(self[0] + '/' + self[1].basename)",
                            "id": "#upload_results_workflow.cwl/upload_fpkm/upload-key"
                        }
                    ],
                    "out": [
                        "#upload_results_workflow.cwl/upload_fpkm/output",
                        "#upload_results_workflow.cwl/upload_fpkm/uuid"
                    ],
                    "id": "#upload_results_workflow.cwl/upload_fpkm"
                },
                {
                    "run": "#bioclient_upload_pull_uuid.cwl",
                    "in": [
                        {
                            "source": "#upload_results_workflow.cwl/bioclient_config",
                            "id": "#upload_results_workflow.cwl/upload_fpkm_uq/config-file"
                        },
                        {
                            "source": "#upload_results_workflow.cwl/htseq_fpkm_uq_file",
                            "id": "#upload_results_workflow.cwl/upload_fpkm_uq/input"
                        },
                        {
                            "source": "#upload_results_workflow.cwl/upload_bucket",
                            "id": "#upload_results_workflow.cwl/upload_fpkm_uq/upload-bucket"
                        },
                        {
                            "source": [
                                "#upload_results_workflow.cwl/job_uuid",
                                "#upload_results_workflow.cwl/htseq_fpkm_uq_file"
                            ],
                            "valueFrom": "$(self[0] + '/' + self[1].basename)",
                            "id": "#upload_results_workflow.cwl/upload_fpkm_uq/upload-key"
                        }
                    ],
                    "out": [
                        "#upload_results_workflow.cwl/upload_fpkm_uq/output",
                        "#upload_results_workflow.cwl/upload_fpkm_uq/uuid"
                    ],
                    "id": "#upload_results_workflow.cwl/upload_fpkm_uq"
                }
            ],
            "id": "#upload_results_workflow.cwl"
        }
    ],
    "cwlVersion": "v1.0"
}
